/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2011 Razor team
 *            2014 LXQt team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Maciej Płaza <plaza.maciej@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * Copyright: 2019 Tianjin KYLIN Information Technology Co., Ltd. *
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QApplication>
#include <QDebug>
#include <QSignalMapper>
#include <QToolButton>
#include <QSettings>
#include <QList>
#include <QMimeData>
#include <QWheelEvent>
#include <QFlag>
#include <QtX11Extras/QX11Info>
#include <QDebug>
#include <QTimer>
#include <XdgDesktopFile>
#include <XdgIcon>
#include <QMessageBox>
#include <QPainterPath>
#include <unistd.h>
#include <sys/utsname.h>
#include "../panel/common/ukuigridlayout.h"

#include "ukuitaskbar.h"
#include "ukuitaskgroup.h"
#include "quicklaunchaction.h"
#define PANEL_SETTINGS     "org.ukui.panel.settings"
#define PANEL_CONFIG_PATH  "/usr/share/ukui/ukui-panel/panel-commission.ini"
#define PANEL_POSITION_KEY  "panelposition"
using namespace UKUi;
/************************************************

************************************************/
UKUITaskBar::UKUITaskBar(IUKUIPanelPlugin *plugin, QWidget *parent) :
    QScrollArea(parent),
    m_signalMapper(new QSignalMapper(this)),
    m_buttonStyle(Qt::ToolButtonIconOnly),
    m_buttonWidth(400),
    m_buttonHeight(100),
    m_showDesktopNum(0),
    m_autoRotate(true),
    m_groupingEnabled(true),
    m_plugin(plugin),
    m_ignoreWindow(),
    m_placeHolder(new QWidget(this)),
    m_style(new LeftAlignedTextStyle()),
    m_maskImg(new QImage()),
    m_beginMaskLabel(new QLabel(this)),
    m_endMaskLabel(new QLabel(this))
{
    m_allFrame=new QWidget(this);
    m_allFrame->setAttribute(Qt::WA_TranslucentBackground);

    this->setWidget(m_allFrame);
    this->horizontalScrollBar()->setVisible(false);
    this->verticalScrollBar()->setVisible(false);
    this->setFrameShape(QFrame::NoFrame);//去掉边框
    this->setWidgetResizable(true);
    //临时方案解决任务栏出现滚动时有滑动条区域遮挡图标,待滚动提示样式确认后再进行替换
    horizontalScrollBar()->setStyleSheet("QScrollBar {height:0px;}");
    verticalScrollBar()->setStyleSheet("QScrollBar {width:0px;}");

    QPalette pal = this->palette();
    pal.setBrush(QPalette::Window, QColor(Qt::transparent));
    this->setPalette(pal);

    m_layout = new UKUi::GridLayout(m_allFrame);
    m_allFrame->setLayout(m_layout);
    m_layout->setMargin(0);
    m_layout->setStretch(UKUi::GridLayout::StretchHorizontal | UKUi::GridLayout::StretchVertical);

    m_placeHolder->setMinimumSize(1,1);
    m_placeHolder->setMaximumSize(QWIDGETSIZE_MAX,QWIDGETSIZE_MAX);
    m_placeHolder->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

    QString filename = QString::fromLocal8Bit(PANEL_CONFIG_PATH);
    QSettings settings(filename, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");
    settings.beginGroup("IgnoreWindow");
    m_ignoreWindow = settings.value("ignoreWindow", "").toStringList();
    settings.endGroup();

    m_interface = new QDBusInterface("com.kylin.kydevmonit.hedronclient","/kydevmonit/hedronclient",
                                    "com.kylin.kydevmonit.hedronclient",
                                    QDBusConnection::systemBus(),this);
    if (m_interface->isValid()) {
        QDBusReply<QString> replay = m_interface->call("get_application_control_mode");
        m_mode = replay;

        QDBusReply<QStringList> list = m_interface->call("get_application_control_list");
        m_controlAppList = list;
    }
    QDBusConnection::systemBus().connect(QString("com.kylin.kydevmonit.hedronclient"),
                                          QString("/com/kylin/kydevmonit/hedron_single"),
                                          QString("com.kylin.kydevmonit.hedronsingle"),
                                          QString("application_control_mode_signal"),
                                          this,
                                          SLOT(securityControlApps(QString))
                                          );


    //往任务栏中加入快速启动按钮
//    refreshQuickLaunch();
    securityControlApps(m_mode);

    settingsChanged();
    setAcceptDrops(true);

    const QByteArray id(PANEL_SETTINGS);
    if (QGSettings::isSchemaInstalled(id)) {
        m_settings=new QGSettings(id);
    }

    const QByteArray id_Theme("org.ukui.style");
    if (QGSettings::isSchemaInstalled(id_Theme)) {
        m_changeTheme = new QGSettings(id_Theme);
    }
    connect(m_changeTheme, &QGSettings::changed, this, [=] (const QString &key) {
                if (key=="iconThemeName") {
                    for(auto it= m_knownSdkWindows.begin(); it != m_knownSdkWindows.end();it++) {
                        UKUITaskGroup *group = it.value();
                        group->updateSdkIcon();
                    }
                }
                if (key == PANEL_POSITION_KEY) {
                    realignSdk();
                }
            });

    connect(m_signalMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped), this, &UKUITaskBar::activateSdkTask);

    connect(WindowManager::self(), &WindowManager::windowChanged, this, &UKUITaskBar::onWindowChanged);
    connect(WindowManager::self(), &WindowManager::windowAdded, this, &UKUITaskBar::onWindowAdded);
    connect(WindowManager::self(), &WindowManager::windowRemoved, this, &UKUITaskBar::onWindowRemoved);
    connect(WindowManager::self(), &WindowManager::currentDesktopChanged, this, &UKUITaskBar::onDesktopChanged);

    saveSdkSettings();

    /**/
    QDBusConnection::sessionBus().unregisterService("com.ukui.panel.plugins.service");
    QDBusConnection::sessionBus().registerService("com.ukui.panel.plugins.service");
    QDBusConnection::sessionBus().registerObject("/taskbar/click", this,QDBusConnection :: ExportAllSlots | QDBusConnection :: ExportAllSignals);

    QDBusConnection::sessionBus().connect(QString(), QString("/taskbar/quicklaunch"), "org.ukui.panel.taskbar", "AddToTaskbar", this, SLOT(addToTaskbar(QString)));
    QDBusConnection::sessionBus().connect(QString(), QString("/taskbar/quicklaunch"), "org.ukui.panel.taskbar", "RemoveFromTaskbar", this, SLOT(removeFromTaskbar(QString)));

    /*监听系统应用的目录以及安卓兼容应用的目录*/
    m_fsWatcher=new QFileSystemWatcher(this);
    m_fsWatcher->addPath(m_desfktopFilePath);
    m_fsWatcher->addPath(m_androidDesktopFilePath);
    directoryUpdated(m_desfktopFilePath);
    directoryUpdated(m_androidDesktopFilePath);
    connect(m_fsWatcher,&QFileSystemWatcher::directoryChanged,[this](){
        directoryUpdated(m_desfktopFilePath);
        directoryUpdated(m_androidDesktopFilePath);
    });

    //xc平板监听平板模式下任务栏固定应用改变
    QDBusConnection::sessionBus().connect("com.kylin.statusmanager.interface","/",
                             "com.kylin.statusmanager.interface","mode_change_signal",
                             this,SLOT(readPanelConfig(bool)));

    if (m_layout->count() == 0) {
        m_layout->addWidget(m_placeHolder);
    } else {
        m_placeHolder->setFixedSize(0,0);
    }

    initMaskUI();
    connect(m_changeTheme, &QGSettings::changed, this, [=] (const QString &key) {
        if (key==STYLE_NAME) {
            initMaskUI();
        }
    });
    const QByteArray id_panelposition(PANEL_SETTINGS);
    if (QGSettings::isSchemaInstalled(id_panelposition)) {
        QGSettings *m_postionGSettings = new QGSettings(id_panelposition);
        connect(m_postionGSettings, &QGSettings::changed, this, [=] (const QString &key) {
            if (key==PANEL_POSITION_KEY) {
                setMaskPostion();
            }
        });
    }
}

/************************************************

 ************************************************/
UKUITaskBar::~UKUITaskBar()
{
    for (auto it = m_vBtn.begin(); it != m_vBtn.end();) {
        (*it)->deleteLater();
        m_vBtn.erase(it);
    }
    m_vBtn.clear();
}

void UKUITaskBar::initMaskUI()
{
    //支持黑白两套主题
    m_maskImg->load("/usr/share/ukui-panel/panel/img/leftMask.svg");
    QColor color = QApplication::palette().color(QPalette::Window);
    color.setAlpha(255);
    for (int x = 0; x <= m_maskImg->width(); x++) {
        color.setAlpha(color.alpha() - 255/m_maskImg->width());
        for (int y = 0; y < m_maskImg->height(); y++) {
            if (color.alpha() > 0) {
                m_maskImg->setPixelColor(x, y, color);
            }
        }
    }
    //初始化遮罩属性
    m_beginMaskLabel->setScaledContents(true);  //设置缩放
    m_beginMaskLabel->setAttribute(Qt::WA_TransparentForMouseEvents, true); //设置鼠标穿透
    m_beginMaskLabel->raise();                 //设置在最顶层
    m_beginMaskLabel->hide();                 //设置默认隐藏
    m_endMaskLabel->setScaledContents(true);
    m_endMaskLabel->setAttribute(Qt::WA_TransparentForMouseEvents, true);
    m_endMaskLabel->raise();
    m_endMaskLabel->hide();

    setMaskPostion();
}

void UKUITaskBar::setMaskPostion()
{
    if(panel()->isHorizontal()) {
        m_beginMaskLabel->setPixmap(QPixmap::fromImage(*m_maskImg));
        m_beginMaskLabel->setGeometry(0, 0, panel()->panelSize(), panel()->panelSize());

        QImage rightimg = m_maskImg->mirrored(true, false);  //水平镜像
        m_endMaskLabel->setPixmap(QPixmap::fromImage(rightimg));
        m_endMaskLabel->setGeometry(this->width() - m_endMaskLabel->width(), 0,
                                    panel()->panelSize(), panel()->panelSize());
    } else {
        QMatrix upmatrix;
        upmatrix.rotate(90.0);         //顺时针旋转90度
        QImage upimg = m_maskImg->transformed(upmatrix,Qt::FastTransformation);
        m_beginMaskLabel->setPixmap(QPixmap::fromImage(upimg));
        m_beginMaskLabel->setGeometry(0, 0, panel()->panelSize(), panel()->panelSize());

        QMatrix bottommatrix;
        bottommatrix.rotate(-90.0);    //逆时针旋转90度
        QImage bottomimg = m_maskImg->transformed(bottommatrix,Qt::FastTransformation);
        m_endMaskLabel->setPixmap(QPixmap::fromImage(bottomimg));
        m_endMaskLabel->setGeometry(0, this->height() - m_endMaskLabel->height(),
                                    panel()->panelSize(), panel()->panelSize());
    }

}

QString UKUITaskBar::readFile(const QString &filename)
{
    QFile f(filename);
    if (!f.open(QFile::ReadOnly)) {
        return QString();
    } else {
        QTextStream in(&f);
        return in.readAll();
    }
}

void UKUITaskBar::onDesktopChanged()
{
    for (auto i = m_knownSdkWindows.begin(); m_knownSdkWindows.end() != i; ++i) {
        (*i)->onDesktopChanged();
        if ((*i)->m_existSameQckBtn) {
            UKUITaskGroup* btn = (*i)->getOwnQckBtn();
            if (m_vBtn.contains(btn))
                btn->setVisible((*i)->isHidden());
        }
    }
}

void UKUITaskBar::readPanelConfig(bool isTabletMode)
{
    if (!isTabletMode) {
        refreshQuickLaunch();
    }
}

void UKUITaskBar::refreshQuickLaunch()
{
    for (auto it = m_vBtn.begin(); it != m_vBtn.end();) {
        (*it)->deleteLater();
        m_vBtn.erase(it);
    }

    QString desktop;
    QString file;

    //gsetting的方式读取写入 apps
    QList<QMap<QString, QVariant> > apps = m_plugin->settings()->readArray("apps");
    QString filename = QDir::homePath() + "/.config/ukui/panel.conf";
    QSettings user_qsettings(filename,QSettings::IniFormat);
    QStringList groupname = user_qsettings.childGroups();
    //为了兼容3.0版本和3.1版本，3.0版本的固定应用信息在quicklaunch里，3.1版本在taskbar里
    //备份还原时需要quicklaunch字段，故不能删除
    if (apps.isEmpty() && groupname.contains("quicklaunch")) {
        apps = copyQuicklaunchConfig();
    }
    addButtonForQuicklanch(apps);
}

QList<QMap<QString, QVariant> > UKUITaskBar::copyQuicklaunchConfig()
{
    QString filename = QDir::homePath() + "/.config/ukui/panel.conf";
    //若taskbar中没有apps，则把quicklaunch中的内容复制到taskbar
    qDebug()<<"Taskbar is empty, read apps from quicklaunch";
    QSettings user_qsettings(filename,QSettings::IniFormat);
    user_qsettings.beginGroup("quicklaunch");
    QList<QMap<QString, QVariant> > array;
    int size = user_qsettings.beginReadArray("apps");
    for (int i = 0; i < size; ++i) {
        user_qsettings.setArrayIndex(i);
        QMap<QString, QVariant> map;
        map["desktop"] = user_qsettings.value("desktop");
        if (array.contains(map)) {
            continue;
        } else {
            array << map;
        }
    }
    user_qsettings.endArray();
    user_qsettings.endGroup();
    user_qsettings.sync();
    return array;
}

void UKUITaskBar::addButtonForQuicklanch(QList<QMap<QString, QVariant> > apps)
{
    for (auto it = m_vBtn.begin(); it != m_vBtn.end();) {
        UKUITaskGroup * quicklaunch = m_vBtn.value(m_vBtn.indexOf(*it));
        if(quicklaunch->m_existSameQckBtn == true) {
            it++;
            continue;
        }
        (*it)->deleteLater();
        m_vBtn.erase(it);
    }

    for (const QMap<QString, QVariant> &app : apps) {
        QString desktop = app.value("desktop", "").toString();
        qDebug()<<"desktop  ******"<<desktop;

        if (!desktop.isEmpty()) {
            XdgDesktopFile xdg;
            if (xdg.load(desktop)) {
                if (!checkButton(new QuickLaunchAction(&xdg, this))) {
                    addSdkButton(new QuickLaunchAction(&xdg, this));
                }
            }
        } else {
            qDebug()<<"error desktop file";
        }
    }
}

void UKUITaskBar::securityControlApps(QString mode)
{
    if (mode == "blacklist") {
        QDBusReply<QStringList> list = m_interface->call("get_application_control_list");
        m_controlAppList = list;
        removeBlackListApp();
    } else if (mode == "whitelist") {
        QDBusReply<QStringList> list = m_interface->call("get_application_control_list");
        m_controlAppList = list;
        addWhiteListApp();
    } else {
        refreshQuickLaunch();
    }
}

void UKUITaskBar::removeBlackListApp()
{
    QList<QMap<QString, QVariant>> blacklistAppnames;
    QStringList mblacklist = m_controlAppList;

    for (int i = 0; i < mblacklist.size(); i++) {
        QMap<QString, QVariant> map;
        map["desktop"] = mblacklist.at(i);
        blacklistAppnames << map;
    }
    QList<QMap<QString, QVariant> > normalapps = m_plugin->settings()->readArray("apps");
    for (const QMap<QString, QVariant> &blacklistAppname : blacklistAppnames) {
        if (normalapps.contains(blacklistAppname)) {
            qDebug()<<"normalapps need remove this:"<<blacklistAppname.value("desktop");
            normalapps.removeOne(blacklistAppname);
        }
    }
    addButtonForQuicklanch(normalapps);
}

void UKUITaskBar::addWhiteListApp()
{
    QList<QMap<QString, QVariant>> whitelistAppnames;
    QStringList mwhitelist = m_controlAppList;

    for (int i = 0; i < mwhitelist.size(); i++) {
        QMap<QString, QVariant> map;
        map["desktop"] = mwhitelist.at(i);
        whitelistAppnames << map;
    }
    addButtonForQuicklanch(whitelistAppnames);
}

bool UKUITaskBar::acceptWindow(const WindowId& window) const
{
    QString platform = QGuiApplication::platformName();
    if(platform.startsWith(QLatin1String("wayland"),Qt::CaseInsensitive)) {

        QString windowTitle = WindowManager::getWindowTitle(window);
        if (windowTitle == "UKUI Panel") {
            return false;
        }
        bool isSkipTaskbar = kdk::WindowManager::skipTaskBar(window);
        if(isSkipTaskbar) {
            return false;
        }
        return true;
    } else {
        int wid = window.toInt();
        QFlags<NET::WindowTypeMask> ignoreList;
        ignoreList |= NET::DesktopMask;
        ignoreList |= NET::DockMask;
        ignoreList |= NET::SplashMask;
        ignoreList |= NET::ToolbarMask;
        ignoreList |= NET::MenuMask;
        ignoreList |= NET::PopupMenuMask;
        ignoreList |= NET::NotificationMask;
        ignoreList |= NET::UtilityMask;

        KWindowInfo info(wid, NET::WMWindowType | NET::WMState, NET::WM2TransientFor);
        if (!info.valid()) {
            return false;
        }
        if (NET::typeMatchesMask(info.windowType(NET::AllTypesMask), ignoreList)) {
            return false;
        }
        if (info.state() & NET::SkipTaskbar) {
            return false;
        }
        // WM_TRANSIENT_FOR hint not set - normal window
        WId transFor = info.transientFor();
        if (transFor == 0 || transFor == wid || transFor == (WId) QX11Info::appRootWindow()) {
            return true;
        }

        info = KWindowInfo(transFor, NET::WMWindowType);
        QFlags<NET::WindowTypeMask> normalFlag;
        normalFlag |= NET::NormalMask;
        normalFlag |= NET::DialogMask;
        normalFlag |= NET::UtilityMask;
        return !NET::typeMatchesMask(info.windowType(NET::AllTypesMask), normalFlag);
    }
}

void UKUITaskBar::buttonMove(UKUITaskGroup * dst, UKUITaskGroup * src, QPoint const & pos)
{
    int src_index;
    if (!src || -1 == (src_index = m_layout->indexOf(src))) {
        return;
    }

    const int size = m_layout->count();
    Q_ASSERT(0 < size);
    //dst is nullptr in case the drop occured on empty space in taskbar
    int dst_index;
    if (nullptr == dst) {
        //moving based on taskbar (not signaled by button)
        QRect occupied = m_layout->occupiedGeometry();
        QRect last_empty_row{occupied};
        const QRect last_item_geometry = m_layout->itemAt(size - 1)->geometry();
        if (m_plugin->panel()->isHorizontal()) {
            if (isRightToLeft()) {
                last_empty_row.setTopRight(last_item_geometry.topLeft());
            } else {
                last_empty_row.setTopLeft(last_item_geometry.topRight());
            }
        } else {
            if (isRightToLeft()) {
                last_empty_row.setTopRight(last_item_geometry.topRight());
            } else {
                last_empty_row.setTopLeft(last_item_geometry.topLeft());
            }
        }
        if (occupied.contains(pos) && !last_empty_row.contains(pos)) {
            return;
        }

        dst_index = size;
    } else {
        //moving based on signal from child button
        dst_index = m_layout->indexOf(dst);
    }

    //moving lower index to higher one => consider as the QList::move => insert(to, takeAt(from))
    if (src_index < dst_index) {
        if (size == dst_index || src_index + 1 != dst_index) {
            --dst_index;
        } else {
            //switching positions of next standing
            const int tmp_index = src_index;
            src_index = dst_index;
            dst_index = tmp_index;
        }
    }

    if (dst_index == src_index || m_layout->animatedMoveInProgress()) {
        return;
    }

    m_layout->moveItem(src_index, dst_index, true);
}

void UKUITaskBar::groupSdkBecomeEmptySlot()
{
    //group now contains no buttons - clean up in hash and delete the group
    UKUITaskGroup * const group = qobject_cast<UKUITaskGroup*>(sender());
    Q_ASSERT(group);

    for (auto i = m_knownSdkWindows.begin(); m_knownSdkWindows.end() != i; ) {
        if (group == *i) {
            m_sdkSwid.removeOne(i.key());
            i = m_knownSdkWindows.erase(i);
            break;
        } else {
            ++i;
        }
    }
    for (auto it = m_vBtn.begin(); it!=m_vBtn.end(); ++it) {
        UKUITaskGroup *pQuickBtn = *it;
        if(pQuickBtn->m_fileName == group->m_fileName
           &&(m_layout->indexOf(pQuickBtn) >= 0 )) {
            pQuickBtn->setHidden(false);
            m_layout->moveItem(m_layout->indexOf(pQuickBtn), m_layout->indexOf(group));
            pQuickBtn->m_existSameQckBtn = false;
            break;
        }
    }
    m_layout->removeWidget(group);
    group->deleteLater();
}

void UKUITaskBar::addWindow(const WindowId& window)
{
    // If grouping disabled group behaves like regular button
    const QString group_id = m_groupingEnabled ? WindowManager::getWindowGroup(window) : QString("%1").arg(window.toInt());
    qDebug()<<"****** group_id is "<<group_id;
    if (m_ignoreWindow.contains(group_id) || group_id == "") {
        return;
    }

    UKUITaskGroup *group = nullptr;
    bool isNeedAddNewWidget = true;
    auto i_group = m_knownSdkWindows.find(window.toInt());
    if (m_knownSdkWindows.end() != i_group) {
        if ((*i_group)->groupName() == group_id) {
            group = *i_group;
        } else {
            (*i_group)->onWindowRemoved(window);
        }
    }

    /*check if window belongs to some existing group
     * 安卓兼容应用的组名为kydroid-display-window
     * 需要将安卓兼容目录的分组特性关闭
    */

    QStringList andriod_window_list;
    andriod_window_list<<"kydroid-display-window"<<"kylin-kmre-window"<<"";
    if (!group && m_groupingEnabled && !andriod_window_list.contains(group_id)) {
        for (auto i = m_knownSdkWindows.cbegin(), i_e = m_knownSdkWindows.cend(); i != i_e; ++i) {
            if ((*i)->groupName() == group_id) {
                group = *i;
                break;
            }
        }
    }
    if (!group) {
        group = new UKUITaskGroup(group_id, window, this);
        connect(group, SIGNAL(groupBecomeEmpty(QString)), this, SLOT(groupSdkBecomeEmptySlot()));
        connect(group, SIGNAL(WindowAddtoTaskBar(QString)), this, SLOT(WindowAddtoTaskBar(QString)));
        connect(group, SIGNAL(WindowRemovefromTaskBar(QString)), this, SLOT(WindowRemovefromTaskBar(QString)));
        connect(group, SIGNAL(visibilityChanged(bool)), this, SLOT(refreshPlaceholderVisibility()));
        connect(group, &UKUITaskButton::dragging, this, [this] (QObject * dragSource, QPoint const & pos) {
                switchButtons(qobject_cast<UKUITaskGroup *>(sender()), qobject_cast<UKUITaskGroup *>(dragSource));//, pos);
                });
        for (auto it = m_vBtn.begin(); it!=m_vBtn.end(); ++it) {
            UKUITaskGroup *pQuickBtn = *it;
            if(pQuickBtn->m_fileName == group->m_fileName
               &&(m_layout->indexOf(pQuickBtn) >= 0 )) {
                m_layout->addWidget(group);
                m_layout->moveItem(m_layout->indexOf(group), m_layout->indexOf(pQuickBtn));
                pQuickBtn->setHidden(true);
                isNeedAddNewWidget = false;
                group->m_existSameQckBtn = true;
                pQuickBtn->m_existSameQckBtn = true;
                group->setQckLchBtn(pQuickBtn);
                break;
            }
        }
        if(isNeedAddNewWidget) {
            m_layout->addWidget(group);
        }
        group->setToolButtonsStyle(m_buttonStyle);
    }
    m_knownSdkWindows[window.toInt()] = group;
    m_sdkSwid.push_back(window.toInt());
    group->addWindow(window);
    group->groupName();
    group->updateSdkIcon();
}

auto UKUITaskBar::removeWindow(windowSdkMap_t::iterator pos) -> windowSdkMap_t::iterator
{
    qint32 const windowid = pos.key();
    UKUITaskGroup * const group = *pos;
    m_sdkSwid.removeOne(windowid);
    auto ret = m_knownSdkWindows.erase(pos);
    WindowId window = WindowId(windowid);
    group->onWindowRemoved(window);
    return ret;
}

void UKUITaskBar::refreshTaskList()
{
    QList<qint32> new_list;
    // Just add new windows to groups, deleting is up to the groups
//    const auto wnds = KWindowSystem::stackingOrder();
    const auto wnds = WindowManager::windows();
    for (int i = 0; i < wnds.size(); i++) {
        if (acceptWindow(wnds.at(i))) {
            new_list << wnds.at(i).toInt();
            addWindow(wnds.at(i));
        }
    }
  //  m_layout->addWidget(m_tmpWidget);

    //emulate windowRemoved if known window not reported by KWindowSystem
    for (auto i = m_knownSdkWindows.begin(), i_e = m_knownSdkWindows.end(); i != i_e; ) {
        if (0 > new_list.indexOf(i.key())) {
            i = removeWindow(i);
        } else {
            ++i;
        }
    }

    refreshPlaceholderVisibility();
}


void UKUITaskBar::onWindowChanged(const WindowId& window)
{
    auto i = m_knownSdkWindows.find(window.toInt());
    if (m_knownSdkWindows.end() != i) {
        if (!(*i)->onWindowChanged(window) && acceptWindow(window)) {
            // window is removed from a group because of class change, so we should add it again
            addWindow(window);
        }
    }
}

void UKUITaskBar::onWindowAdded(const WindowId& window)
{
    qDebug()<<"window    is :  ******"<<window;
    auto const pos = m_knownSdkWindows.find(window.toInt());
    if (m_knownSdkWindows.end() == pos && acceptWindow(window)) {
        addWindow(window);
    }
}

void UKUITaskBar::onWindowRemoved(const WindowId& window)
{
    auto const pos = m_knownSdkWindows.find(window.toInt());
    if (m_knownSdkWindows.end() != pos) {
        removeWindow(pos);
    }
}

void UKUITaskBar::refreshButtonRotation()
{
    bool autoRotate = m_autoRotate && (m_buttonStyle != Qt::ToolButtonIconOnly);

    IUKUIPanel::Position panelPosition = m_plugin->panel()->position();
}

/************************************************

 ************************************************/
void UKUITaskBar::refreshPlaceholderVisibility()
{
    bool haveVisibleWindow = false;
    for (auto i = m_knownSdkWindows.cbegin(), i_e = m_knownSdkWindows.cend(); i_e != i; ++i) {
        if ((*i)->isVisible()) {
            haveVisibleWindow = true;
            break;
        }
    }
    m_placeHolder->setVisible(!haveVisibleWindow);
    if (haveVisibleWindow || m_layout->count() != 0) {
        m_placeHolder->setFixedSize(0,0);
     } else {
        m_placeHolder->setMinimumSize(1,1);
        m_placeHolder->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
    }
}

/************************************************

 ************************************************/
void UKUITaskBar::setButtonStyle(Qt::ToolButtonStyle buttonStyle)
{
    emit buttonStyleRefreshed(m_buttonStyle);
}

void UKUITaskBar::settingsChanged()
{
    bool groupingEnabledOld = m_groupingEnabled;

    m_buttonWidth = m_plugin->settings()->value("buttonWidth", 400).toInt();
    m_buttonHeight = m_plugin->settings()->value("buttonHeight", 100).toInt();
    QString s = m_plugin->settings()->value("buttonStyle").toString().toUpper();

    if (s == "ICON") {
        setButtonStyle(Qt::ToolButtonIconOnly);
    } else if (s == "TEXT") {
        setButtonStyle(Qt::ToolButtonTextOnly);
    } else {
        setButtonStyle(Qt::ToolButtonIconOnly);
    }

    m_showDesktopNum = m_plugin->settings()->value("showDesktopNum", m_showDesktopNum).toInt();
    m_autoRotate = m_plugin->settings()->value("autoRotate", true).toBool();
    m_groupingEnabled = m_plugin->settings()->value("groupingEnabled",true).toBool();

    // Delete all groups if grouping feature toggled and start over
    if (groupingEnabledOld != m_groupingEnabled) {
        for (int i = m_knownSdkWindows.size() - 1; 0 <= i; --i) {
            UKUITaskGroup * group = m_knownSdkWindows.value(m_sdkSwid.value(i));
            if (nullptr != group) {
                m_layout->takeAt(i);
                group->deleteLater();
            }
        }
        m_knownSdkWindows.clear();
        m_sdkSwid.clear();
    }

    refreshTaskList();
}

void UKUITaskBar::realignSdk()
{
    m_layout->setEnabled(false);
    refreshButtonRotation();

    IUKUIPanel *panel = m_plugin->panel();
    QSize btnSize = QSize(m_plugin->panel()->panelSize(), m_plugin->panel()->panelSize());
    int iconsize = panel->iconSize();

    bool rotated = false;
    if (panel->isHorizontal()) {
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        this->setMinimumHeight(panel->panelSize());
        m_allFrame->setMinimumSize(QSize((m_layout->count())*panel->panelSize(),panel->panelSize()));
        if (m_allFrame->width() < this->width()) {
            m_allFrame->setFixedWidth(this->width());
        }

        m_layout->setRowCount(panel->lineCount());
        m_layout->setColumnCount(0);

        m_beginMaskLabel->setGeometry(0, 0, panel->panelSize(), panel->panelSize());
        m_endMaskLabel->setGeometry(this->width() - m_endMaskLabel->width(), 0,
                                      panel->panelSize(), panel->panelSize());
        if(m_layout->count() * panel->panelSize() > this->width()) {
            m_endMaskLabel->show();
        } else {
            m_beginMaskLabel->hide();
            m_endMaskLabel->hide();
        }
    } else {
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        this->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        this->setMinimumWidth(panel->panelSize());
        m_allFrame->setMinimumSize(QSize(panel->panelSize(),(m_layout->count())*panel->panelSize()));
        if (m_allFrame->height() < this->height()) {
            m_allFrame->setFixedHeight(this->height());
        }

        m_layout->setRowCount(0);

        if (m_buttonStyle == Qt::ToolButtonIconOnly) {
            // Vertical + Icons
            m_layout->setColumnCount(panel->lineCount());
        } else {
            rotated = m_autoRotate && (panel->position() == IUKUIPanel::PositionLeft || panel->position() == IUKUIPanel::PositionRight);

            // Vertical + Text
            if (rotated) {
                m_layout->setColumnCount(panel->lineCount());
            } else {
                m_layout->setColumnCount(1);
            }
        }
        m_beginMaskLabel->setGeometry(0, 0, panel->panelSize(), panel->panelSize());
        m_endMaskLabel->setGeometry(0, this->height() - m_endMaskLabel->height(),
                                       panel->panelSize(), panel->panelSize());
        if(m_layout->count() * panel->panelSize() > this->height()) {
            m_endMaskLabel->show();
        } else {
            m_beginMaskLabel->hide();
            m_endMaskLabel->hide();
        }
    }

    for(auto it= m_knownSdkWindows.begin(); it != m_knownSdkWindows.end();it++) {
        UKUITaskGroup *group = it.value();
        group->setIconSize(QSize(iconsize,iconsize));
    }
    for (int i = 0; i < m_vBtn.size(); i++) {
        UKUITaskGroup * quicklaunch = m_vBtn.value(i);
        quicklaunch->setIconSize(QSize(iconsize, iconsize));
    }
    m_layout->setCellFixedSize(btnSize);
    m_layout->setEnabled(true);
    //our placement on screen could have been changed
    emit showOnlySettingChanged();
    emit refreshIconGeometry();

    if (panel->isHorizontal()) {
        horizontalScrollBar()->setMaximum(m_allFrame->width() - this->width());
    } else {
        verticalScrollBar()->setMaximum(m_allFrame->height() - this->height());
    }
}

void UKUITaskBar::wheelEvent(QWheelEvent* event)
{
    if (this->verticalScrollBarPolicy()==Qt::ScrollBarAlwaysOff) {
        if (event->delta()>=0) {
            horizontalScrollBar()->setValue(horizontalScrollBar()->value()-40);
        } else {
            horizontalScrollBar()->setValue(horizontalScrollBar()->value()+40);
            if (horizontalScrollBar()->value()>m_allFrame->width()) {
                horizontalScrollBar()->setValue(m_allFrame->width());
            }
        }

        if(horizontalScrollBar()->value() != 0) {
            m_beginMaskLabel->show();
        } else {
            m_beginMaskLabel->hide();
        }
        if (horizontalScrollBar()->value() == horizontalScrollBar()->maximum()) {
            m_endMaskLabel->hide();
        } else {
            m_endMaskLabel->show();
        }
    } else {
        if (event->delta()>=0) {
            verticalScrollBar()->setValue(verticalScrollBar()->value()-40);
        } else {
            verticalScrollBar()->setValue(verticalScrollBar()->value()+40);
        }
        if(verticalScrollBar()->value() != 0) {
            m_beginMaskLabel->show();
        } else {
            m_beginMaskLabel->hide();
        }
        if (verticalScrollBar()->value() == verticalScrollBar()->maximum()) {
            m_endMaskLabel->hide();
        } else {
            m_endMaskLabel->show();
        }
    }
}

void UKUITaskBar::resizeEvent(QResizeEvent* event)
{
    emit refreshIconGeometry();
    return QWidget::resizeEvent(event);
}

void UKUITaskBar::changeEvent(QEvent* event)
{
    // if current style is changed, reset the base style of the proxy style
    // so we can apply the new style correctly to task buttons.
    if(event->type() == QEvent::StyleChange)
        m_style->setBaseStyle(NULL);

    QFrame::changeEvent(event);
}

void UKUITaskBar::activateSdkTask(int pos)
{
    for (int i = 0; i < m_knownSdkWindows.size(); ++i) {
        UKUITaskGroup * g = m_knownSdkWindows.value(m_sdkSwid.value(i));
        if (g && g->isVisible()) {
            pos--;
            if (pos == 0) {
                g->raiseSdkApplication();
                break;
            }
        }
    }
}

void UKUITaskBar::enterEvent(QEvent *)
{
    update();
}

void UKUITaskBar::leaveEvent(QEvent *)
{
    update();
}

void UKUITaskBar::mouseMoveEvent(QMouseEvent *e)
{
    QFrame::mouseMoveEvent(e);
}


void UKUITaskBar::mousePressEvent(QMouseEvent *)
{
    QDBusMessage message =QDBusMessage::createSignal("/taskbar/click", "com.ukui.panel.plugins.taskbar", "sendToUkuiDEApp");
    QDBusConnection::sessionBus().send(message);
}

/// About Quick Launch Functions

void UKUITaskBar::addSdkButton(QuickLaunchAction* action)
{
    bool isNeedAddNewWidget = true;
    m_layout->setEnabled(false);
    UKUITaskGroup *btn = new UKUITaskGroup(action, m_plugin, this);
    btn->setArrowType(Qt::NoArrow);
    /*＠bug
     * 快速启动栏右键菜单原本的样式有对于不可选项有置灰效果，
     * 后跟随主题框架之后置灰效果消失，可能与此属性相关
     */
    for (auto it = m_knownSdkWindows.begin(); it!=m_knownSdkWindows.end(); ++it) {
        UKUITaskGroup *group = *it;
        if(btn->m_fileName == group->m_fileName
           &&(m_layout->indexOf(group) >= 0)) {
            m_layout->addWidget(btn);
            m_layout->moveItem(m_layout->indexOf(btn), m_layout->indexOf(group));
            isNeedAddNewWidget = false;
            group->m_existSameQckBtn = true;
            btn->m_existSameQckBtn = true;
            m_vBtn.push_back(btn);
            group->setQckLchBtn(btn);
            btn->setHidden(group->isVisible());
            break;
        }
    }
    if (isNeedAddNewWidget) {
        m_layout->addWidget(btn);
        btn->setIconSize(QSize(m_plugin->panel()->iconSize(),m_plugin->panel()->iconSize()));
        m_vBtn.push_back(btn);
        m_layout->moveItem(m_layout->indexOf(btn), countOfButtons() - 1);
    }
    connect(btn, &UKUITaskButton::dragging, this, [this] (QObject * dragSource, QPoint const & pos) {
            switchButtons(qobject_cast<UKUITaskGroup *>(sender()), qobject_cast<UKUITaskGroup *>(dragSource));//, pos);
            });
    connect(btn, SIGNAL(buttonDeleted()), this, SLOT(buttonSdkDeleted()));
    m_layout->setEnabled(true);
}

void UKUITaskBar::switchButtons(UKUITaskGroup *dst_button, UKUITaskGroup *src_button)
{
    if (dst_button == src_button) {
        return;
    }
    if (!dst_button || !src_button) {
        return;
    }

    int dst = m_layout->indexOf(dst_button);
    int src = m_layout->indexOf(src_button);
    if (dst == src || m_layout->animatedMoveInProgress()) {
        return;
    }
    m_layout->moveItem(src, dst, true);
    saveSdkSettings();
}

bool UKUITaskBar::checkButton(QuickLaunchAction* action)
{
    bool checkresult;
    UKUITaskGroup* btn = new UKUITaskGroup(action, m_plugin, this);
    int i = 0;
    int counts = m_vBtn.size();
    if (m_vBtn.size()>0) {
        while (i != counts) {
            UKUITaskGroup *b = m_vBtn.value(i);
            qDebug()<<"m_layout->itemAt("<<i<<") ";
            if(b->m_fileName == btn->m_fileName) {
                checkresult=true;
                break;
            } else {
                checkresult=false;
                ++i;
            }
        }
        delete btn;
        return checkresult;
    } else {
        qDebug()<<"countOfButtons =  "<<countOfButtons();
        delete btn;
        return false;
    }
}

/* 以xdg的方式删除　button*/
void UKUITaskBar::removeButton(QuickLaunchAction* action)
{
    int i = 0;
    UKUITaskGroup* btn = new UKUITaskGroup(action, m_plugin, this);
    if (!btn) {
        return;
    }
    while (i < m_vBtn.size()) {
        UKUITaskGroup *tmp = m_vBtn.value(i);
        if (QString::compare(btn->m_fileName, tmp->m_fileName) == 0) {
            doInitGroupButton(tmp->m_fileName);
            tmp->deleteLater();
            m_layout->removeWidget(tmp);
            m_vBtn.remove(i);
            break;
        }
        ++i;
    }

    btn->deleteLater();
    saveSdkSettings();
}

void UKUITaskBar::removeButton(QString file)
{
    int i = 0;
    while (i < m_vBtn.size()) {
        UKUITaskGroup *tmp = m_vBtn.value(i);
        if (QString::compare(file, tmp->m_fileName) == 0) {
            doInitGroupButton(tmp->m_fileName);
            tmp->deleteLater();
            m_layout->removeWidget(tmp);
            m_vBtn.remove(i);
            break;
        }
        ++i;
    }
}

void UKUITaskBar::WindowAddtoTaskBar(QString arg)
{
    for(auto it= m_knownSdkWindows.begin(); it != m_knownSdkWindows.end();it++) {
        UKUITaskGroup *group = it.value();
            if (arg.compare(group->groupName()) == 0) {
                addSdkToTaskbar(group->m_fileName);
                realignSdk();
                break;
        }
    }
}

void UKUITaskBar::WindowRemovefromTaskBar(QString arg)
{
    for (auto it = m_vBtn.begin(); it!=m_vBtn.end(); ++it) {
        UKUITaskGroup *pQuickBtn = *it;
        if(pQuickBtn->m_fileName == arg && (m_layout->indexOf(pQuickBtn) >= 0 )) {
            doInitGroupButton(pQuickBtn->m_fileName);
            m_vBtn.removeOne(pQuickBtn);
            pQuickBtn->deleteLater();
            m_layout->removeWidget(pQuickBtn);
            saveSdkSettings();
            break;
        }
    }
}

// 只要任何监控的目录更新（添加、删除、重命名），就会调用。
void UKUITaskBar::directoryUpdated(const QString &path)
{
    // 比较最新的内容和保存的内容找出区别(变化)
    QStringList currentrylist = m_currentContentsMap[path];
    const QDir dir(path);
    QStringList newentrylist = dir.entryList(QDir::NoDotAndDotDot  | QDir::AllDirs | QDir::Files, QDir::DirsFirst);
    QSet<QString> newdirset = QSet<QString>::fromList(newentrylist);
    QSet<QString> currentdirset = QSet<QString>::fromList(currentrylist);

    // 文件已被移除
    QSet<QString> deletedfiles = currentdirset - newdirset;
    QStringList deletefile = deletedfiles.toList();

    // 更新当前设置
    m_currentContentsMap[path] = newentrylist;

    // 从Dir中删除文件/目录
    if (!deletefile.isEmpty()) {
        foreach (QString file, deletefile) {
            removeButton(path+file);
            saveSdkSettings();
        }
    }
}

void UKUITaskBar::addToTaskbar(QString arg)
{
    const auto url=QUrl(arg);
    QString fileName(url.isLocalFile() ? url.toLocalFile() : url.url());
    XdgDesktopFile xdg;
    if (xdg.load(fileName)) {
        if (!checkButton(new QuickLaunchAction(&xdg, this))) {
            addSdkButton(new QuickLaunchAction(&xdg, this));
            m_placeHolder->hide();
        }
    } else {
        qWarning() << "XdgDesktopFile" << fileName << "is not valid";
    }
    saveSdkSettings();
    realignSdk();
}

void UKUITaskBar::addSdkToTaskbar(QString arg)
{
    const auto url=QUrl(arg);
    QString fileName(url.isLocalFile() ? url.toLocalFile() : url.url());
    QFileInfo fi(fileName);
    XdgDesktopFile xdg;
    if (xdg.load(fileName)) {
        if (!checkButton(new QuickLaunchAction(&xdg, this))) {
            addSdkButton(new QuickLaunchAction(&xdg, this));
            m_placeHolder->hide();
        }
    } else {
        qWarning() << "XdgDesktopFile" << fileName << "is not valid";
        QMessageBox::information(this, tr("Drop Error"),
                                 tr("File/URL '%1' cannot be embedded into QuickLaunch for now").arg(fileName)
                                 );
    }
    saveSdkSettings();
}

void UKUITaskBar::removeFromTaskbar(QString arg)
{
    XdgDesktopFile xdg;
    xdg.load(arg);
    removeButton(new QuickLaunchAction(&xdg, this));
      realignSdk();
}

void UKUITaskBar::doInitGroupButton(QString sname)
{
    for(auto it= m_knownSdkWindows.begin(); it != m_knownSdkWindows.end();it++) {
        UKUITaskGroup *group = it.value();
        if (group->m_existSameQckBtn) {
            if (sname == group->m_fileName) {
                    group->m_existSameQckBtn = false;
                    group->setQckLchBtn(NULL);
                    break;
            }
        }
    }
}

void UKUITaskBar::buttonSdkDeleted()
{
    UKUITaskGroup *btn = qobject_cast<UKUITaskGroup*>(sender());
    if (!btn) {
        return;
    }
    for (auto it = m_vBtn.begin(); it != m_vBtn.end(); it++) {
        if (*it == btn) {
            for (auto it= m_knownSdkWindows.begin(); it != m_knownSdkWindows.end(); it++) {
                UKUITaskGroup *group = it.value();
                if (group->m_existSameQckBtn) {
                    if (btn->m_fileName == group->m_fileName) {
                            group->m_existSameQckBtn = false;
                            group->setQckLchBtn(NULL);
                    }
                }
            }
            m_vBtn.erase(it);
            break;
        }
    }
    m_layout->removeWidget(btn);
    btn->deleteLater();
    saveSdkSettings();
}

void UKUITaskBar::saveSdkSettingsSlot()
{
    saveSdkSettings();
}

void UKUITaskBar::saveSdkSettings()
{
    PluginSettings *settings = m_plugin->settings();
    settings->remove("apps");
    QList<QMap<QString, QVariant>> hashList;
    int size = m_layout->count();
    for (int j = 0; j < size; ++j) {
        UKUITaskGroup *b = qobject_cast<UKUITaskGroup*>(m_layout->itemAt(j)->widget());
        if (!(m_vBtn.contains(b) || m_knownSdkWindows.contains(m_knownSdkWindows.key(b)))) {
            continue;
        }
        if (!b->m_statFlag && b->m_existSameQckBtn) {
            continue;
        }
        if (!b) {
            continue;
        }
        if (b->m_statFlag && b->m_existSameQckBtn) {
            b = b->getQckLchBtn();
        }
        if (!b || b->m_statFlag) {
            continue;
        }
        // convert QHash<QString, QString> to QMap<QString, QVariant>
        QMap<QString, QVariant> map;
        QHashIterator<QString, QString> it(b->settingsMap());
        while (it.hasNext()) {
            it.next();
            map[it.key()] = it.value();
        }
        hashList << map;
    }
    settings->setArray("apps", hashList);
}

int UKUITaskBar::countOfButtons() const
{
    return m_layout->count();
}
