/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2011 Razor team
 *            2014 LXQt team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * Copyright: 2019 Tianjin KYLIN Information Technology Co., Ltd. *
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#ifndef UKUITASKBUTTON_H
#define UKUITASKBUTTON_H
//panel
#include "../panel/iukuipanel.h"
#include "quicklaunchaction.h"
#include "../panel/customstyle.h"
#include "../panel/common/common.h"
//Qt
#include <QGSettings/QGSettings>
#include <QToolButton>
#include <QMimeData>
//SDK
#include <windowmanager/windowmanager.h>
using namespace kdk;

class QPainter;
class QPalette;
class QMimeData;
class UKUITaskGroup;
class UKUITaskBar;
class IUKUIPanelPlugin;

class LeftAlignedTextStyle : public QProxyStyle
{
    using QProxyStyle::QProxyStyle;
public:

    virtual void drawItemText(QPainter * painter, const QRect & rect, int flags
            , const QPalette & pal, bool enabled, const QString & text
            , QPalette::ColorRole textRole = QPalette::NoRole) const override;
};


class UKUITaskButton : public QToolButton
{
    Q_OBJECT

    Q_PROPERTY(Qt::Corner origin READ origin WRITE setOrigin)

public:
    explicit UKUITaskButton(const QString & groupName, const WindowId& window, UKUITaskBar * taskBar, QWidget *parent = 0);
    UKUITaskButton(QuickLaunchAction * act, IUKUIPanelPlugin * plugin, QWidget* parent = 0);
    virtual ~UKUITaskButton();

    WindowId sdkWindowId() const { return m_sdkWindow; }

    bool hasUrgencyHint() const { return m_urgencyHint; }
    void setUrgencyHint(bool set);
    void setSdkUrgencyHint(bool set);
    bool isOnDesktop(int desktop) const;
    bool isOnCurrentScreen() const;
    bool isMinimized() const;
    bool isSdkMinimized() const;
    bool isSdkFocusState() const;
    void updateSdkText();
    void setLeaderWindow(const WindowId& leaderWindow);
    bool isLeaderWindow(const WindowId& compare) { return m_sdkWindow == compare; }

    Qt::Corner origin() const;
    virtual void setAutoRotation(bool value, IUKUIPanel::Position position);

    UKUITaskBar * parentTaskBar() const {return m_parentTaskBar;}

    void refreshIconGeometry(QRect const & geom);
    static QString mimeDataFormat() { return QLatin1String("ukui/UKUITaskButton"); }
    /*! \return true if this buttom received DragEnter event (and no DragLeave event yet)
     * */

    QHash<QString,QString> settingsMap();
    QString m_fileName;

    void toDoModifyQuicklaunchMenuAction(bool direction) { modifyQuicklaunchMenuAction(direction);}

    inline IUKUIPanelPlugin * plugin() const { return m_plugin; }

public slots:
    void raiseSdkApplication();
    void minimizeSdkApplication();
    void closeApplication();
    void setOrigin(Qt::Corner);
    void updateSdkIcon();
    void selfRemove();
    void customContextMenuRequestedSlot(const QPoint & pos);
    void setGroupIcon(QIcon ico);
    void setTaskButtonVisible(bool visible);


protected:
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dragMoveEvent(QDragMoveEvent * event);
    virtual void dragLeaveEvent(QDragLeaveEvent *event);
    virtual void dropEvent(QDropEvent *event);
    void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void enterEvent(QEvent *);
    void leaveEvent(QEvent *);
    void paintEvent(QPaintEvent *);

    virtual QMimeData * mimeData();
    static bool m_draggging;

private:
    enum TaskButtonStatus{NORMAL, HOVER, PRESS};

    bool m_statFlag = true;
    bool m_urgencyHint = false;

    WId m_window;
    WindowId m_sdkWindow;
    QPoint m_dragStartPosition;
    Qt::Corner m_origin;
    UKUITaskBar * m_parentTaskBar;
    IUKUIPanelPlugin * m_plugin;
    TaskButtonStatus m_taskButtonStatus;
    QIcon m_icon;

    // Timer for when draggind something into a button (the button's window
    // must be activated so that the use can continue dragging to the window
    QTimer * m_DNDTimer;
    QGSettings *m_gsettings;

    ///////////////////////////////////
    QuickLaunchAction *m_act;
    QAction *m_deleteAct;
    QMenu *m_menu;
    QGSettings *m_gsettingsQuickLaunch;

    void modifyQuicklaunchMenuAction(bool direction);
private slots:
    void activateSdkWithDraggable();


signals:
    void dropped(QObject * dragSource, QPoint const & pos);
    void dragging(QObject * dragSource, QPoint const & pos);
    //////////////////////////////////
    void buttonDeleted();
};


class ButtonMimeData: public QMimeData
{
    Q_OBJECT
public:
    ButtonMimeData():
        QMimeData(),
        m_button(0)
    {
    }

    UKUITaskButton *button() const { return m_button; }
    void setButton(UKUITaskButton *button) { m_button = button; }

private:
    UKUITaskButton *m_button;
};
typedef QHash<qint64, UKUITaskButton*> UKUISdkTaskButtonHash;

#endif // UKUITASKBUTTON_H
