/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2011 Razor team
 *            2014 LXQt team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Maciej Płaza <plaza.maciej@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * Copyright: 2019 Tianjin KYLIN Information Technology Co., Ltd. *
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef UKUITASKGROUP_H
#define UKUITASKGROUP_H
//panel
#include "../panel/iukuipanel.h"
#include "../panel/iukuipanelplugin.h"
#include "../panel/pluginsettings.h"
#include "../panel/common/common.h"
#include "ukuitaskbar.h"
#include "ukuigrouppopup.h"
#include "ukuitaskbutton.h"
//QT
#include <QTimer>
#include <QApplication>
#include <QScreen>
//KDE
#include <KF5/KWindowSystem/kwindowsystem.h>

class QVBoxLayout;
class IUKUIPanelPlugin;

class UKUIGroupPopup;
class UKUiMasterPopup;

class UKUITaskGroup: public UKUITaskButton
{
    Q_OBJECT

public:
    const int previewWindowWidth =  (QApplication::screens().at(0)->size().width() / 5);
    const int previewWindowHeight = (QApplication::screens().at(0)->size().height() / 5);
    bool m_statFlag = true;
    bool m_existSameQckBtn = false;
    UKUITaskGroup(const QString & groupName, const WindowId& window, UKUITaskBar * parent);
    UKUITaskGroup(QuickLaunchAction * act, IUKUIPanelPlugin * plugin, UKUITaskBar *parent);
    virtual ~UKUITaskGroup();
    QString groupName() const { return m_groupName; }
    UKUITaskGroup* getOwnQckBtn() { return this->m_qckLchBtn; }
    UKUITaskGroup* getQckLchBtn() { return m_qckLchBtn; }
    void setQckLchBtn(UKUITaskGroup *utgp) { if(m_statFlag) m_qckLchBtn = utgp; }

    UKUITaskButton * addWindow(const WindowId& id);
    bool onWindowChanged(const WindowId& window);
    void setToolButtonsStyle(Qt::ToolButtonStyle style);
    void setPopupVisible(bool visible = true, bool fast = false);
    void showPreviewMode();
    void singleSdkWindowClick();
    void setAutoRotation(bool value, IUKUIPanel::Position position);
    int visibleSdkButtonsCount() const;

public slots:
    void onWindowRemoved(const WindowId& window);
    void timeout();
    void onDesktopChanged();

protected:
    QMimeData * mimeData();

    void leaveEvent(QEvent * event);
    void enterEvent(QEvent * event);
    void dragEnterEvent(QDragEnterEvent * event);
    void dragLeaveEvent(QDragLeaveEvent * event);
    void contextMenuEvent(QContextMenuEvent * event);
    void mouseMoveEvent(QMouseEvent * event);
    void mouseReleaseEvent(QMouseEvent *event);
    int recalculateFrameWidth() const;

    void paintEvent(QPaintEvent *);

private slots:
    void onClicked(bool checked);
    void onActiveWindowChanged(const WindowId& window);

    void closeGroup();
    void refreshIconsGeometry();
    void refreshSdkVisibility();
    void handleSavedEvent();
    void AddtoTaskBar();
    void RemovefromTaskBar();

signals:
    void groupBecomeEmpty(QString name);
    void visibilityChanged(bool visible);
    void WindowAddtoTaskBar(QString arg);
    void WindowRemovefromTaskBar(QString arg);

private:
    enum TaskGroupStatus{NORMAL, HOVER, PRESS};
    enum TaskGroupEvent{ENTEREVENT, LEAVEEVENT, OTHEREVENT};

    UKUITaskBar * m_parent;
    UKUITaskGroup *m_qckLchBtn;
    QString m_groupName;
    UKUIGroupPopup * m_popup;
    UKUISdkTaskButtonHash m_sdkButtonHash;
    UKUISdkTaskButtonHash m_sdkVisibleHash;
    TaskGroupStatus m_taskGroupStatus;
    TaskGroupEvent  m_taskGroupEvent;
    QEvent * m_event;
    QTimer *m_timer;
    QSize recalculateFrameSize();
    QPoint recalculateFramePosition();
    bool m_preventPopup;

    void recalculateFrameIfVisible();
    void initSdkDesktopFileName(const WindowId& window);
    void initActionsInRightButtonMenu();
    void changeTaskButtonStyle();

    ///////////////////////////////
    // quicklaunch button
    QuickLaunchAction *m_act;
    IUKUIPanelPlugin * m_plugin;
    QAction *m_deleteAct;
    QMenu *m_menu;
    QGSettings *m_gsettings;

    int m_allWidgetWidth = 0;
    int m_allWidgetHeight = 0;
    bool m_isShowByList = false;
    bool m_isThumbnailAvailable = false;
    void getThumbnailIfAvailable();

    void rightMenuCloseAction(QMenu *menu);

};

#endif // UKUITASKGROUP_H
