/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */


#ifndef UKUITASKWIDGET_H
#define UKUITASKWIDGET_H

#include <QProxyStyle>
#include "../panel/iukuipanel.h"
#include <QtX11Extras/qtx11extrasversion.h>
#include <QLabel>
#include <QVBoxLayout>
#include <QStyle>
#include <KWindowInfo>
#include <QProcess>

#include <QtQuick/QQuickView>
#include <QUrl>
#include <QMainWindow>
#include <QtQuickWidgets/QtQuickWidgets>
#include <QGSettings>

#include <windowmanager/windowmanager.h>
using namespace kdk;

class QPainter;
class QPalette;
class QMimeData;
class UKUITaskGroup;
class UKUITaskBar;
class UKUITaskCloseButton;

class UKUITaskWidget : public QWidget
{
    Q_OBJECT
public:
    explicit UKUITaskWidget(const WindowId& window, UKUITaskBar * taskBar, QWidget *parent = 0);
    virtual ~UKUITaskWidget();

    bool isApplicationActive() const;
    WindowId sdkWindowId() const { m_sdkWindow;}

    void setThumbScale(bool val);
    void setThumbMaximumSize(int w);
    void updateSdkText();

    UKUITaskBar * parentTaskBar() const {return m_parentTaskBar;}

    static QString mimeDataFormat() { return QLatin1String("ukui/UKUITaskWidget"); }
    /*! \return true if this buttom received DragEnter event (and no DragLeave event yet)
     * */
    bool hasDragAndDropHover() const;
    void setThumbNail(QPixmap _pixmap);
    void setLiveThumbNail(int w, int h);
    void removeLiveThumbNail();
    void setTitleFixedWidth(int size);
    void updateTitle();
    void removeThumbNail();
    void addThumbNail();
    void setPixmap(QPixmap _pixmap);
    void setIcon(QIcon ico);

    QString getAndroidAppIcon(const WindowId& windowId);
    bool canUseOpenGL() { return m_canUseOpenGL;}

public slots:
    void raiseSdkApplication();
    void minimizeApplication();
    void maximizeApplication();
    void deMaximizeApplication();

    void closeSdkApplication();
    /**
     * @brief setWindowKeepAbove
     * 窗口置顶
     */
    void setWindowKeepAbove();
    /**
     * @brief setWindowStatusClear
     * 取消置顶
     */
    void setWindowStatusClear();

    void updateSdkIcon();

protected:
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void enterEvent(QEvent *);
    void leaveEvent(QEvent *);
    void paintEvent(QPaintEvent *);
    void contextMenuEvent(QContextMenuEvent *event);

    void setWindowId(WId wid) {m_window = wid;}
    virtual QMimeData * mimeData();
    inline IUKUIPanelPlugin * plugin() const { return m_plugin; }

private:
    WId m_window;
    WindowId m_sdkWindow;
    QPixmap m_pixmap;
    UKUITaskBar * m_parentTaskBar;
    IUKUIPanelPlugin * m_plugin;
    QLabel *m_titleLabel;
    QLabel *m_thumbnailLabel;
    QLabel *m_appIcon;
    QIcon m_icon;
    UKUITaskCloseButton *m_closeBtn;
    QVBoxLayout *m_vWindowsLayout;
    QHBoxLayout *m_topBarLayout;

    QProcess *m_cmdProcess;
    QWidget *m_thumbnailPreview;
    QQuickView *m_tempView;
    QObject *m_tempItem;
    QObject *m_tempRecl;
    bool m_canUseOpenGL = true;

    QGSettings *m_transparencyGsettings;
    double m_transparency;
    void getSysTransparency();

    // Timer for when draggind something into a button (the button's window
    // must be activated so that the use can continue dragging to the window
    QTimer * m_DNDTimer;
    enum TaskWidgetStatus{NORMAL, HOVER, PRESS};
    TaskWidgetStatus m_status;

    QTimer *m_timer;
    QGSettings *m_changeTheme;

private slots:
    void activateSdkWithDraggable();

    void keepShow();
    void keepLeave();

    void sendSigToKwin();
    void sendCloseSigToKWin();
    void rightButtonMenu();

signals:
    void dropped(QObject * dragSource, QPoint const & pos);
    void dragging(QObject * dragSource, QPoint const & pos);
    void windowActived();
    void closeSigtoPop();
    void closeSigtoGroup();
    void signalKeepShow(bool flag);
    void signalKeepLeave(bool flag);
};

typedef QHash<WId,UKUITaskWidget*> UKUITaskWidgetHash;
typedef QHash<qint64, UKUITaskWidget*> UKUISdkTaskWidgetHash;

#endif // UKUITASKWIDGET_H
